@@ -43,20 +43,59 @@ namespace sieve_util {
             fprintf(stderr, "LARGE LIMIT=%.2e\n", limit);
             limit = MAX_LIMIT;
         }
-        assert(limit > 1000);
-        return limit;
+        return std::max((uint64_t) limit, 2UL);
     }
 
 
     std::vector<char> sieve(mpz_t &N, uint64_t gap, uint64_t limit, size_t &prime_count) {
-        // Sieve
-        std::vector<char> composite(gap + 1, 0);
+        // 10GB would be a lot ram.
+        if ((gap < 0) || (gap > (1L << 40))) { return {}; }
+        if (limit > (1L << 50)) { return {}; }
 
-        // Handle all evens
-        uint64_t first = mpz_cdiv_ui(N, 2);
-        for (uint32_t d = first; d <= gap; d += 2) {
+        // Sieve of [N, N+gap]
+        gap += 1;
+        std::vector<char> composite(gap, 0);
+
+        // primes >= N_int, should avoid marking themselves off.
+        uint64_t N_int = mpz_cmp_ui(N, limit) <= 0 ? mpz_get_ui(N) : limit + 1;
+        // only needed when limit > N
+        uint64_t N_end = N_int + gap;
+
+        // Mark 0 as composite because.
+        if ((N_int == 0) && (N_end >= 0)) {
+            composite[0] = true;
+        }
+
+        // Mark 1 as composite because.
+        if ((N_int <= 1) && (N_end >= 1)) {
+            composite[1 - N_int] = true;
+        }
+
+        // limit doesn't need to exceed sqrt(N + gap)
+        if (mpz_sizeinbase(N, 2) < 128) {
+            mpz_t temp;
+            mpz_init(temp);
+
+            mpz_add_ui(temp, N, gap);
+            mpz_sqrt(temp, temp);
+            if (mpz_cmp_ui(temp, limit) < 0) {
+                if (!mpz_fits_ulong_p(temp)) { return {}; }
+                limit = mpz_get_ui(temp);
+            }
+
+            mpz_clear(temp);
+        }
+
+        if (limit > N_end) { return {}; }
+
+        // Remove all evens
+        for (uint32_t d = mpz_cdiv_ui(N, 2); d < gap; d += 2) {
             composite[d] = true;
         }
+        if ((N_int <= 2) && (N_end >= 2)) {
+            // Go back and mark 2 as prime
+            composite[2 - N_int] = false;
+        }
 
         prime_count = 1;
         primes::iterator iter;
@@ -66,38 +105,42 @@ namespace sieve_util {
         assert(prime == 3);
 
         // small primes can divide multiple numbers
-        for (; prime <= gap && prime <= limit; prime = iter.next()) {
+        for (; prime < gap && prime <= limit; prime = iter.next()) {
             prime_count++;
-            uint64_t first = mpz_cdiv_ui(N, prime);
-            if (first % 2 == 1) {
-                first += prime;
-            }
 
-            for (uint32_t d = first; d <= gap; d += 2 * prime) {
+            uint64_t two_p = 2 * prime;
+            uint64_t first = mpz_cdiv_ui(N, two_p);
+            first += prime;
+            if (first >= two_p) first -= two_p;
+
+            for (uint32_t d = first; d < gap; d += two_p) {
                 composite[d] = true;
             }
+
+            if (N_int <= prime) {
+                // mark prime as prime
+                composite[prime - N_int] = false;
+            }
         }
 
-        // TODO make sure limit * limit < N
-
-        /* Think about returning this for debug or something
-        size_t unknowns = std::count(composite.begin(), composite.end(), false);
-        size_t count_c = gap - unknowns;
-        fprintf(stderr, "%ld / %ld = %.2f composite, %ld remaining (primes %ld)\n",
-                gap - unknowns, gap, 100.0 * count_c / gap, unknowns, prime_count);
-        */
-
         for (; prime <= limit; prime = iter.next()) {
             prime_count++;
-            uint64_t first = mpz_cdiv_ui(N, prime);
-            if ((first < gap) && ((first & 1) == 0)) {
-                composite[first / 2] = true;
+
+            // Only one in the interval
+            // either N_int <= prime (the number is the prime)
+            // or N_int > prime (and we mark off some multiple
+            if (N_int > prime) {
+
+                uint64_t two_p = 2 * prime;
+                uint64_t first = mpz_cdiv_ui(N, two_p);
+                first += prime;
+                if (first >= two_p) first -= two_p;
+                if (first < gap) {
+                    composite[first] = true;
+                }
             }
         }
 
-        assert(composite[0] == false);
-        assert(composite[gap] == false);
-
         // Possible a vector copy, but fast in the overall scheme of things.
         return composite;
     }
